<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Application\Form\Shop\Type;

use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\Invite\InviteInput;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\DataMapperInterface;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Security;
use Symfony\Component\Validator\Constraints\Email;

class ReferralInviteType extends AbstractType implements DataMapperInterface
{
    /** @var Security */
    private $security;

    public function __construct(
        Security $security
    ) {
        $this->security = $security;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->setDataMapper($this)
            ->add('email', EmailType::class, [
                'label' => 'sylius.form.customer.email',
                'constraints' => [
                    new Email([
                        'groups' => ['Default', 'sylius'],
                    ]),
                ],
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => InviteInput::class,
        ]);
    }

    public function mapDataToForms($viewData, $forms): void
    {
        $forms = iterator_to_array($forms);

        $forms['email']->setData($viewData ? $viewData->getEmail() : '');
    }

    public function mapFormsToData($forms, &$viewData): void
    {
        $forms = iterator_to_array($forms);

        $viewData = InviteInput::create(
            $this->security->getUser(),
            $forms['email']->getData(),
        );
    }
}
