<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Infrastructure\Email;

use Dedi\SyliusCustomerReferralPlugin\Domain\Common\Email\EmailInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Common\Email\EmailSenderInterface;
use Sylius\Component\Mailer\Sender\SenderInterface;

class EmailSender implements EmailSenderInterface
{
    /** @var SenderInterface */
    private $sender;

    /** @var string */
    private $codePrefix;

    public function __construct(
        SenderInterface $sender,
        string $codePrefix
    ) {
        $this->sender = $sender;
        $this->codePrefix = $codePrefix;
    }

    public function send(EmailInterface ...$emails): void
    {
        foreach ($emails as $email) {
            $this->sender->send(
                $this->getCode($email),
                $email->getRecipients(),
                $email->getData()
            );
        }
    }

    private function getCode(
        EmailInterface $email
    ): string {
        return sprintf('%s_%s', $this->codePrefix, $email->getCode());
    }
}
