<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Infrastructure\Repository;

use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\InviteeAlreadyInvitedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Model\ReferralModelInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Repository\ReferralRepositoryInterface;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;

class ReferralRepository extends EntityRepository implements ReferralRepositoryInterface
{
    public function save(ReferralModelInterface $referralModel): ReferralModelInterface
    {
        try {
            $this->add($referralModel);

            return $referralModel;
        } catch (UniqueConstraintViolationException $e) {
            throw new InviteeAlreadyInvitedException();
        }
    }

    public function findByInviteeEmail(?string $inviteeEmail): ?ReferralModelInterface
    {
        /** @var ReferralModelInterface $referral */
        $referral = $this->findOneBy([
            'inviteeEmail' => $inviteeEmail,
        ]);

        return $referral;
    }

    public function findByCode(string $code): ?ReferralModelInterface
    {
        [$id, $inviteeEmail] = explode(':', base64_decode($code));

        /** @var ReferralModelInterface $referral */
        $referral = $this->findOneBy([
            'id' => $id,
            'inviteeEmail' => $inviteeEmail,
        ]);

        return $referral;
    }
}
